package jwars;

import java.util.Arrays;
import java.util.Iterator;

/**
 * This class contains the main method of JWars.
 */
public class Main {

  private static void printVersion(){
    System.out.println("This is "+Util.NAME_AND_VERSION);
    System.out.println(""+Util.GAME_NAME + " is an acronym for "+Util.FULL_GAME_NAME);
    System.out.println("Written by "+Util.AUTHORS);
  }

  private static void printHelp(){
    System.out.println("Parameters: [-ofhv]");
    System.out.println();
    System.out.println("o: enable opengl pipeline");
    System.out.println("f: run in full-screen mode");
    System.out.println("h: print this help and exit");
    System.out.println("v: print version information and exit");
  }

  private static void parseChars(String arg){
    for(int i = 1; i < arg.length(); i++){
      switch(arg.charAt(i)){
        case 'o':
          System.out.println("Attempting to enable Java2D OpenGL pipeline");
          System.setProperty("sun.java2d.opengl", "True");
          break;
        case 'f':
          System.out.println("Enabling pseudo-fullscreen mode");
          Util.isFullScreen=true;
          break;
        case 'v':
          printVersion();
          System.exit(0);
          break;
        case 'h':
          printVersion();
          System.out.println();
          printHelp();
          System.exit(0);
        default:
          System.out.println("Unrecognized option: "+arg.charAt(i));
          System.out.println();
          printHelp();
          System.exit(-1);
      }
    }
  }

  private static void parseArgs(Iterator<String> args){
    while(args.hasNext()){
      String arg = args.next();
      String[] prop = arg.split("=");
      if(prop.length != 2){
        throw new IllegalArgumentException("Illegal argument format. Must be on the form key=value");
      }
      setProperty(prop);
    }
  }

  private static void setProperty(String[] prop){
    if(prop[0].equals("name")){
      Util.setUserName(prop[1]);
    }
  }

  public static void main(String[] args){
    System.out.println("Initializing JWars");
    System.out.println("Currently running JVM version: "+
                       System.getProperty("java.runtime.version"));
    Iterator<String> argIterator = Arrays.asList(args).iterator();
    if(args.length > 0 && args[0].startsWith("-")){
      parseChars(argIterator.next());
    }
    parseArgs(argIterator);
    new WarMenu();
  }

}
