package jwars.control;

import java.util.ArrayList;

import jwars.model.map.CoordinateSystem;

/**
 * Class for managing the focus point on the map.
 * NOTE: should probably be moved to the GUI package
 *
 */
public class ViewPort{

  private int portWidth, portHeight;
  private int currentLocationX, currentLocationY;
  private final CoordinateSystem coords;

  private ArrayList<ViewPortListener> listeners =
      new ArrayList<ViewPortListener>();

  public ViewPort(CoordinateSystem coords, int startX, int startY,
                  int portWidth, int portHeight){
    this.coords = coords;
    this.portWidth = portWidth;
    this.portHeight = portHeight;
    setLocation(startX, startY);
  }

  public int getCurrentX(){
    return currentLocationX;
  }

  public int getCurrentY(){
    return currentLocationY;
  }

  private void setLocationWithoutCompensation(int x, int y){
    x = (x < 0 ? 0 : x); //Restrict sign and magnitude of arguments
    y = (y < 0 ? 0 : y); //Nicer than 16 lines of if-else
    x = (x + portWidth >= coords.WIDTH ? coords.WIDTH - portWidth - 1 : x);
    y = (y + portHeight >= coords.HEIGHT ? coords.HEIGHT - portHeight - 1 : y);

    this.currentLocationX = x;
    this.currentLocationY = y;
    for(ViewPortListener v : listeners){
      v.reportLocationChange(x, y);
    }

  }

  public void setSize(int width, int height){
    portWidth = width;
    portHeight = height;
  }

  public void setLocation(int x, int y){
    x -= portWidth / 2; //Compensation for origin in corner
    y -= portHeight / 2;
    setLocationWithoutCompensation(x,y);
  }

  public void nudge(int dx, int dy){
    setLocationWithoutCompensation(this.currentLocationX + dx,
                                   this.currentLocationY + dy);
  }

  public void addListener(ViewPortListener listener){
    listeners.add(listener);
  }

  public static interface ViewPortListener{

    public void reportLocationChange(int x, int y);

  }

}
