package jwars.event;



public class EventQueue{

  private Event firstElement, lastElement;
  private int currentTime;
  private final int delay;

  /**
   * Create an event queue which executes events at the specified time after
   * insertion
   * @param delay int delay of execution measured in frames after insertion
   */
  public EventQueue(int delay){
    if(delay < 0){
      throw new RuntimeException("Cannot use negative delay: "+delay);
    }
    this.delay = delay;
  }

  /**
   * Execute all events in this queue which are due for this frame
   * @param time current frame count
   */
  public void update(int time){
    currentTime = time;
    if(firstElement != null && firstElement.time < time){
      System.err.println("Time stamp out of date!");
      throw new RuntimeException(
          "Time stamp out of date in [" + this +"] : " + firstElement.time
          + " - current time: " + time + "; Event: " + firstElement);
    }
    Event currentElement;
    while (firstElement != null && firstElement.time == time){
      currentElement = firstElement;
      firstElement = firstElement.next;
      currentElement.time = Event.notScheduled;
      currentElement.next = null;
      currentElement.execute();
    }
  }

  /**
   * Append argument Event to the end of this queue.
   * @param event
   */
  protected void add(Event event){
    if(event.time != Event.notScheduled)
      throw new RuntimeException("Event "+event+" already in a queue");
    event.time = currentTime + delay + 1; //Set execution time.
    event.next = null;
    if(firstElement == null)
      firstElement = lastElement = event;
    else{
      lastElement.next = event;
      lastElement = event;
    }
  }

  public String toString(){
    return super.toString() + "; Queue Delay="+delay;
  }

}
