package jwars.event;

import java.util.ArrayList;
import java.util.List;

/**
 * Keeps a number of Queues registered, and updates them when the update method
 * of this QueueManager is called.
 */
public class QueueManager{

  //Can be used for keystrokes and other instantaneous input
  //which originates from any Thread.
  private final SynchronizedQueue IOQueue = new SynchronizedQueue();
  //private final EventQueue instantQueue = new EventQueue(0);
  private final List<EventQueue> queues = new ArrayList<EventQueue>();

  /**
   * Poll all registered queues.
   * @param time int
   */
  public void update(int time){
    IOQueue.update(time);
    //instantQueue.update(time);
    for(EventQueue eq : queues){
      eq.update(time);
    }
  }

  /**
   * Register the argument EventQueue in this QueueManager.
   * @param queue queue to be added
   */
  public void register(EventQueue queue){
    queues.add(queue);
  }

  /**
   * Returns a syncrhonized EventQueue implementation which
   * can be used for keystrokes and other instantaneous input
   * which originates from any Thread. Successive calls will return the same
   * EventQueue.
   * @return SynchronizedQueue
   */
  public SynchronizedQueue getInstantQueue(){
    return IOQueue;
  }

}
