package jwars.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.image.BufferedImage;
import javax.swing.JPanel;

import jwars.control.ViewPort;
import jwars.model.Unit;
import jwars.model.WarModel;
import jwars.model.map.CoordinateSystem;
import jwars.model.map.TerrainTile;
import jwars.model.map.TileMap;

public class MapPanel
    extends JPanel implements ViewPort.ViewPortListener{

  private WarPanel wPanel;
  private int x, y;
  private CoordinateSystem mapCoords; // = new CoordinateSystem(WarModel.pixelCoords,
  public final int size;
  private final Color grassColor = Color.green.darker().darker();
  private final Color forestColor = grassColor.darker().darker().darker().darker();
  int grassGreenValue = grassColor.getGreen();
  int forestGreenValue = forestColor.getGreen();
  private BufferedImage backGround;

  //private int xOffset, yOffset; //If the map is smaller than the panel,
  //these variables are used to align the graphics

  public MapPanel(WarPanel wPanel, int size){
    this.wPanel = wPanel;
    this.size = size;
    //Presently only works for SQUARE coordinate systems
    //this.mapCoords = new CoordinateSystem(WarModel.PIXEL_COORDS, WarModel.PIXEL_COORDS.WIDTH/size);
    this.mapCoords = CoordinateSystem.createLargestCompatibleSystem(WarModel.
        PIXEL_COORDS, size, size);
    this.setPreferredSize(new Dimension(mapCoords.WIDTH, mapCoords.HEIGHT));
    MapListener list = new MapListener();
    wPanel.VIEWPORT.addListener(this);
    this.addMouseListener(list);
    this.addMouseMotionListener(list);

    //Make background image
    backGround = jwars.Util.createImage(mapCoords.WIDTH, mapCoords.HEIGHT, java.awt.Transparency.OPAQUE);
    Graphics g = backGround.getGraphics();
    TileMap<TerrainTile> terrain = WarModel.WORLD.TERRAIN_MAP;
    int maxForestation = 0;
    for(int i = 0; i < mapCoords.WIDTH; i++){
      for(int j = 0; j < mapCoords.HEIGHT; j++){
        int x = mapCoords.transformToCoords(terrain, i);
        int y = mapCoords.transformToCoords(terrain, j);
        TerrainTile tile = terrain.getTile(x,y);
        int forestation = 0;
        //The null-check shouldn't be necessary, remove when non-square systems are properly supported
        if(tile != null)
          forestation = tile.forestation;
        if(forestation > maxForestation){
          maxForestation = forestation;
        }
        int green = grassGreenValue + (int)(forestation/(float)maxForestation * (forestGreenValue - grassGreenValue));
        g.setColor(new Color(0, green, 0));
        g.drawLine(i,j,i,j);
      }
    }
    g.dispose();
    backGround = jwars.Util.recreate(backGround);
    //this.setBackground(Color.black);
    //innerPanel.setPreferredSize(new Dimension(mapCoords.WIDTH, mapCoords.HEIGHT));
    //this.add(innerPanel);
    //this.setBorder(BorderFactory.createEtchedBorder());

  }

  public void reportLocationChange(int x, int y){
    int width = WarModel.PIXEL_COORDS.transformToCoords(mapCoords,
        wPanel.getWidth());
    int height = WarModel.PIXEL_COORDS.transformToCoords(mapCoords,
        wPanel.getHeight());

    //int oldX = this.x;
    //int oldY = this.y;

    this.x = WarModel.PIXEL_COORDS.transformToCoords(mapCoords, x);
    this.y = WarModel.PIXEL_COORDS.transformToCoords(mapCoords, y);

    //int xMin = Math.min(oldX, x);
    //int yMin = Math.min(oldY, y);
    //int xMax = Math.max(oldX, x) + width;
    //int yMax = Math.max(oldY, y) + height;

    //paintImmediately(xMin-1, yMin-1, xMax-xMin, yMax-yMin);
    render();
    //paintImmedi(oldX, oldY, width+1, height+1);
    //repaint(x-1, y-1, width+1, height+1);
  }

  public void render(){
    //System.out.println(""+this.getWidth());
    //paintImmediately(0, 0, this.getWidth(), this.getHeight());
    repaint();
  }

  protected void paintComponent(Graphics g){
    super.paintComponent(g);
    //g.translate(xOffset,yOffset);
    //System.out.println("wPanel width "+wPanel.WIDTH);
    int width = WarModel.PIXEL_COORDS.transformToCoords(mapCoords,
        wPanel.getWidth());
    int height = WarModel.PIXEL_COORDS.transformToCoords(mapCoords,
        wPanel.getHeight());

    g.drawImage(backGround, 0, 0, null);
    //System.out.println(""+width);
    //The following two lines can be done simpler using CoordinateSystem API
    //int width =  wPanel.WIDTH*size*unitsPerPixel/WarModel.mainCoords.WIDTH;
    //int height = wPanel.HEIGHT*size*unitsPerPixel/WarModel.mainCoords.HEIGHT;
    //g.setColor(this.grassColor);
    //g.fillRect(0, 0, mapCoords.WIDTH, mapCoords.HEIGHT);
    g.setColor(Color.black);
    g.drawRect(x, y, width, height);


    g.setColor(Color.red);
    drawUnit(g, WarModel.WORLD.getUnit(0));
  }

  public void drawUnit(Graphics g, Unit u){
    int x = WarModel.WORLD.MAIN_COORDS.transformToCoords(mapCoords,u.getX());
    int y = WarModel.WORLD.MAIN_COORDS.transformToCoords(mapCoords,u.getY());
    g.drawLine(x, y, x, y);
    for(int i = 0; i < u.getSubUnitCount(); i++){
      drawUnit(g, u.getUnit(i));
    }
  }

  private class MapListener
      implements MouseListener, MouseMotionListener{

    private void setPosition(int x, int y){
      //x = mapCoords.localToMainCoords(x);
      //y = mapCoords.localToMainCoords(y);
      //wPanel.VIEWPORT.setLocation(x,y);
      x = mapCoords.transformToCoords(WarModel.PIXEL_COORDS, x);
      y = mapCoords.transformToCoords(WarModel.PIXEL_COORDS, y);
      wPanel.VIEWPORT.setLocation(x, y);
    }

    public void mousePressed(MouseEvent e){
      int button = e.getButton();
      if(button == MouseEvent.BUTTON1){
        setPosition(e.getX(), e.getY());
      } else if(button == MouseEvent.BUTTON3){
        int x = mapCoords.transformToCoords(WarModel.WORLD.MAIN_COORDS, e.getX());
        int y = mapCoords.transformToCoords(WarModel.WORLD.MAIN_COORDS, e.getY());
        wPanel.orderMove(e, x, y);
      }
    }

    public void mouseDragged(MouseEvent e){
      if(!e.isMetaDown())
        setPosition(e.getX(), e.getY());
    }

    public void mouseMoved(MouseEvent e){}

    public void mouseClicked(MouseEvent e){}

    public void mouseReleased(MouseEvent e){}

    public void mouseEntered(MouseEvent e){}

    public void mouseExited(MouseEvent e){}
  }

  /*private class InnerPanel
      extends JPanel{

    protected void paintComponent(Graphics g){
      super.paintComponent(g);
      //g.translate(xOffset,yOffset);
      //System.out.println("wPanel width "+wPanel.WIDTH);
      int width = WarModel.PIXEL_COORDS.transformToCoords(mapCoords,
          wPanel.getWidth());
      int height = WarModel.PIXEL_COORDS.transformToCoords(mapCoords,
          wPanel.getHeight());
      //System.out.println(""+width);
      //The following two lines can be done simpler using CoordinateSystem API
      //int width =  wPanel.WIDTH*size*unitsPerPixel/WarModel.mainCoords.WIDTH;
      //int height = wPanel.HEIGHT*size*unitsPerPixel/WarModel.mainCoords.HEIGHT;
      g.setColor(grassColor);
      g.fillRect(0, 0, mapCoords.WIDTH, mapCoords.HEIGHT);
      g.setColor(Color.black);
      g.drawRect(x, y, width, height);
    }

  }*/

}
