package jwars.gui;

import java.util.Enumeration;
import java.util.Hashtable;

import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.TreePath;

import jwars.control.Control;
import jwars.control.UnitSelectionListener;
import jwars.model.Unit;
import jwars.model.UnitListener;
import jwars.model.WarModel;

public class OverViewPanel extends JTabbedPane implements UnitListener, UnitSelectionListener, TreeSelectionListener {

  private JTree unitTree;
  private Hashtable<Unit, DefaultMutableTreeNode> nodes = new Hashtable<Unit, DefaultMutableTreeNode>();

  private int defaultExpansionLevel = 3;

  public OverViewPanel(){
    WarModel.WORLD.addUnitListener(this);
    Control.UNIT_SELECTION.addListener(this);
    Unit root = WarModel.WORLD.getEveryOne();
    registerTree(root);
    unitTree = new JTree(nodes.get(root));
    setFocusable(false);
    unitTree.setFocusable(false);
    JScrollPane scroller = new JScrollPane(unitTree,
        JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
        JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
    add("ORBAT",scroller);
    add("Score", new JLabel("42\n List of players: blahblah", JLabel.CENTER));
    unitTree.setCellRenderer(createRenderer());
    unitTree.addTreeSelectionListener(this);
    expandToDefault(unitTree.getPathForRow(0));
  }

  public void unitNowOnBoard(Unit u){
    register(u);
  }

  public void registerTree(Unit u){
    register(u);
    for(int i = 0; i < u.getSubUnitCount(); i++){
      registerTree(u.getUnit(i));
    }
  }

  public void register(Unit u){
    DefaultMutableTreeNode node = new DefaultMutableTreeNode(u);
    nodes.put(u, node);
    Unit superUnit = u.getSuperUnit();
    DefaultMutableTreeNode parent = null;
    if(superUnit != null){
      parent = nodes.get(superUnit);
    }
    if(parent != null){
      parent.add(node);
    }
  }

  private TreeCellRenderer createRenderer(){
    DefaultTreeCellRenderer rend = new DefaultTreeCellRenderer();
    return rend;
  }

  private void expandToDefault(TreePath path){
    if(path.getPathCount() > defaultExpansionLevel){
      return;
    }
    DefaultMutableTreeNode node = (DefaultMutableTreeNode)path.getLastPathComponent();
    unitTree.expandPath(path);
    int count = node.getChildCount();
    //int count = unitTree.getRowCount()
    for(int i = 0; i < count; i++){
      TreePath newPath = new TreePath(((DefaultMutableTreeNode)node.getChildAt(i)).getPath());
      expandToDefault(newPath);
    }
    //unitTree.expandPath(unitTree.);
  }

  private void deselect(Unit unit){
    unitTree.clearSelection();
    //System.out.println("deselecting "+unit);
    DefaultMutableTreeNode node = nodes.get(unit);
    TreePath path = new TreePath(((DefaultMutableTreeNode)node.getParent()).getPath());
    //TreePath path = new TreePath(node.getPath());
    //System.out.println("will collapse path for "+unit);
    //System.out.println(""+path);
    //unitTree.collapsePath(path);
    collapseAll(path);
    //unitTree.clearSelection();
  }

  /*private void collapseChildren(TreePath path){
    //Enumeration e = unitTree.getExpandedDescendants()
    unitTree.getExpandedDescendants(path);
  }*/

  private void collapseAll(TreePath path){
    collapseChildren(path);
    collapseParents(path);
  }

  private void collapseChildren(TreePath path){
    Enumeration<TreePath> descendants = unitTree.getExpandedDescendants(path);
    while(descendants.hasMoreElements()){
      TreePath desc = descendants.nextElement();
      if(desc.getPathCount() > this.defaultExpansionLevel)
        unitTree.collapsePath(desc);
    }
  }

  private void collapseParents(TreePath path){
    if(path.getPathCount() <= this.defaultExpansionLevel){
      return;
    }
    unitTree.collapsePath(path);
    TreePath parent = path.getParentPath();
    if(parent != null){
      collapseParents(parent);
    }
  }

  public void unitSelected(Unit newSelection, Unit lastSelection){
    if(lastSelection != null){
      deselect(lastSelection);
    }
    if(newSelection == null){
      unitTree.clearSelection();
      return;
    }
    DefaultMutableTreeNode newNode = nodes.get(newSelection);
    TreePath newSelectionPath = new TreePath(newNode.getPath());
    unitTree.getSelectionModel().setSelectionPath(newSelectionPath);
    unitTree.expandPath(newSelectionPath);
  }

  public void valueChanged(TreeSelectionEvent e){
    DefaultMutableTreeNode node = (DefaultMutableTreeNode)
                       unitTree.getLastSelectedPathComponent();
    if (node == null){
      return;
    }
    Unit unit = (Unit)node.getUserObject();
    if(Control.UNIT_SELECTION.markedUnit() != unit){
      Control.UNIT_SELECTION.markUnit(unit);
    int x = WarModel.WORLD.MAIN_COORDS.transformToCoords(WarModel.PIXEL_COORDS, unit.getX());
    int y = WarModel.WORLD.MAIN_COORDS.transformToCoords(WarModel.PIXEL_COORDS, unit.getY());
    Control.VIEWPORT.setLocation(x, y);
    }

  }

}
