package jwars.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.JOptionPane;

import jwars.control.Community;
import jwars.test.ConnectionListener;
import jwars.test.Player;

public class PlayerList extends JList implements ConnectionListener {

  private DefaultListModel model;

  public PlayerList(){
    super(new DefaultListModel());
    this.model = (DefaultListModel)getModel();
    this.setCellRenderer(new PlayerCellRenderer());
    Community.players.addConnectionListener(this);

    MouseListener mouseListener = new MouseAdapter() {
      public void mouseClicked(MouseEvent e) {
        int index = locationToIndex(e.getPoint());
        Player p = (Player)model.elementAt(index);
        if(Community.players.isLocalPlayer(p)){
          if(e.getButton() == MouseEvent.BUTTON1){
            p.setTeam(p.getTeam().opponent());
            Community.playerPropertiesChanged();
          }
          else if(e.getButton() == MouseEvent.BUTTON3){
            String name = JOptionPane.showInputDialog("Enter nickname:");
            if(name != null){
              p.setName(name);
              Community.playerPropertiesChanged();
            }
          }
        }
      }
    };
    addMouseListener(mouseListener);

  }

  public void playerJoined(Player player){
    model.addElement(player);
  }

  public void playerLeft(Player player){
    model.removeElement(player);
  }

  public void playerPropertyChanged(Player player){
    repaint();
  }

  private class PlayerCellRenderer extends DefaultListCellRenderer {

    public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
      super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
      Player p = (Player)value;
      setIcon(p.getTeam().FLAG);
      Color color = Community.players.isLocalPlayer(p) ? Color.blue : Color.black;
      setForeground(color);
      return this;
    }

  }

}
