package jwars.model;

import jwars.event.QueueManager;
import jwars.model.map.CollisionTile;
import jwars.model.map.CoordinateSystem;
import jwars.model.map.MapFactory;
import jwars.model.map.TerrainTile;
import jwars.model.map.TileMap;

/**
 * This class contains constants and singleton constructs of JWars. It is
 * candidate for replacement with a more object oriented structure.
 */
public class WarModel{

  private final static int mainCoordinateSystemSize = 1 << 22; //about 8e6
  private final static int unitsPerPixel = 1 << 9;
  private final static int terrainTileSizeInPixels = 1 << 5;
  private final static int collisionMapSize = 1 << 7;
  private final static int collisionTileSize =
      mainCoordinateSystemSize / collisionMapSize;

  public final static int TIME_PER_FRAME = 20; //milliseconds

  private final static CoordinateSystem MAIN_COORDS = new CoordinateSystem(
      mainCoordinateSystemSize, mainCoordinateSystemSize);

  public final static CoordinateSystem PIXEL_COORDS =
      new CoordinateSystem(MAIN_COORDS, unitsPerPixel);

  private final static TileMap<TerrainTile> TERRAIN_MAP =
      MapFactory.createTerrainMap(PIXEL_COORDS, terrainTileSizeInPixels);

  private final static TileMap<CollisionTile> COLLISION_MAP =
      MapFactory.createCollisionMap(MAIN_COORDS, collisionTileSize);

  final static double meter = PIXEL_COORDS.transformToCoords(MAIN_COORDS, 3);
  final static double second = 1d;

  public final static World WORLD =
      new World(MAIN_COORDS, COLLISION_MAP, TERRAIN_MAP, meter, second);

  public final static QueueManager QUEUES = new QueueManager();

}
