package jwars.model.ai.impl;

import static jwars.Util.minusPiToPi;
import jwars.model.Collider;
import jwars.model.Moveable;
import jwars.model.Unit;
import jwars.model.ai.Attackable;
import jwars.model.ai.ListeningAI;
import jwars.model.ai.UnitAI;
import jwars.model.weapons.ReloadListener;
import jwars.model.weapons.Weapon;

public class MoveableAI
    implements UnitAI, ReloadListener {

  private final Moveable owner;
  private MovementQueueAI queueAI;
  private BasicAttacker attackAI;

  public MoveableAI(Moveable owner){
    this.owner = owner;
    queueAI = new MovementQueueAI(owner){
      public void setDestination(int x, int y){
        MoveableAI.this.setDestination(x, y);
      }

    };
    attackAI = new BasicAttacker(owner);
  }

  public void reportFinished(Object source){
  }

  public void reportFailed(Object source){
  }

  public void orderMove(ListeningAI source, int x, int y){
    queueAI.orderMove(source, x, y);
  }

  public void appendMove(ListeningAI source, int x, int y){
    queueAI.appendMove(source, x, y);
  }

  public void prependMove(ListeningAI source, int x, int y){
    queueAI.prependMove(source, x, y);
  }

  public void orderHalt(ListeningAI source){
    queueAI.orderHalt(source);
  }

  public void setDestination(int x, int y){
    owner.setDestination(x, y);
  }

  public boolean isMoving(){
    return queueAI.isMoving();
  }

  public boolean closeEnough(int tolerance){
    return queueAI.closeEnough(tolerance);
  }

  public double getTargetAngle(){
    return attackAI.getTargetAngle();
  }

  public double getMoveAngle(){
    return queueAI.getMoveAngle();
  }

  public void walkAround(Collider other){
    int otherX = other.getX();
    int otherY = other.getY();
    int thisX = owner.getX();
    int thisY = owner.getY();
    int dx = otherX - thisX;
    int dy = otherY - thisY;
    double otherDirection = Math.atan2(dx, dy);
    double walkingDirection = Math.atan2(owner.getDestinationX() - thisX,
        owner.getDestinationY() - thisY);
    double rightOrLeft = minusPiToPi(walkingDirection - otherDirection);

    //double rightOrLeft = Math.signum((double)destination.x * (double)dy - (double)destination.y * (double)dx);
    int newDirX, newDirY;
    if(rightOrLeft > 0){
      newDirX = dy;
      newDirY = -dx;
    } else{
      newDirX = -dy;
      newDirY = dx;
    }
    int targetX = thisX + newDirX;
    int targetY = thisY + newDirY;
    if(queueAI.currentAutority() == navigationAI){
      queueAI.changeNextWayPoint(navigationAI, targetX, targetY);
    } else{
      prependMove(navigationAI, targetX, targetY);
    }

  }

  public Unit owner(){
    return owner;
  }

  /**
   * Used to distinguish between legit orders and pathfinding. Orders that are
   * issued by this object are only used by the pathfinder.
   */
  private ListeningAI navigationAI = new ListeningAI(){
    public void reportFinished(Object source){
    }

    public void reportFailed(Object source){
    }
  };

  public void orderAttack(Attackable target){
    attackAI.orderAttack(target);
  }

  public void orderAttack(int x, int y, int radius){
    attackAI.orderAttack(x, y, radius);
  }

  public void orderHoldFire(){
    attackAI.orderHoldFire();
  }

  public boolean isFiring(){
    return attackAI.isFiring();
  }

  public boolean reportReloaded(Weapon weapon){
    //attackAI.reportReloaded();
    return false;
    //return owner.getTheta() == attackAI.getTargetAngle();
  }

  public int getTargetX(){
    return attackAI.getTargetX();
  }

  public int getTargetY(){
    return attackAI.getTargetY();
  }

  /*public static void main(String[] args){
    for(int i = -50; i < 50; i++){
      double m = i / 4d;
      if(Math.abs(minusPiToPi(m)) >= Math.PI){
        System.out.println("ERROR");
      }
    }
     }*/

}
