package jwars.model.map;

import java.util.Random;

import jwars.model.Coord;
import jwars.model.World;
import jwars.model.structures.BluePrint;
import jwars.model.structures.PolygonStructure;
import jwars.model.structures.TerrainObject;
import java.awt.image.BufferedImage;
import java.awt.Graphics;

public class StructureFactory{

  public static void buildDefaultStructure(World world){
    BluePrint bp = new BluePrint(true, 150);
    BufferedImage im = new BufferedImage(30,30,java.awt.Transparency.OPAQUE);
    Graphics g = im.createGraphics();
    g.setColor(java.awt.Color.cyan);
    g.fillRect(10,10,12,6);
    TerrainObject to = new TerrainObject(im, bp, 2, world.metric(600), world.metric(1000));
    TerrainObject to2 = new TerrainObject(im, bp, 2, (200), (200));
    world.DETECTOR.register(to);
  }





  public static void buildDefaultStructures(World world){
    int cityX = (int)(world.MAIN_COORDS.WIDTH * 0.42);
    int cityY = (int)(world.MAIN_COORDS.HEIGHT * 0.42);
    BluePrint[] prints = randomBluePrints(7, world.metric(10), world.metric(25));

    //int width = world.metric(40);
    //int height = world.metric(30);
    //PolygonStructure johnsHouse = new PolygonStructure(standardHouse(width,height), 1d, world.metric(200),world.metric(300));
    //world.DETECTOR.register(johnsHouse);
    //world.DETECTOR.register();
    //BluePrint[] prints = randomBluePrints
    PolygonStructure[] city = city(prints, 16, cityX, cityY, world.metric(256));
    for(PolygonStructure p : city){
      world.DETECTOR.register(p);
    }
  }

  private final static Random generator = new Random(1);
  //private final static int minSize = 10;
  //private final static int maxSize = 35;
  //private final static int spread = maxSize-minSize;

  public static Coord[] rectangle(int width, int height){
    Coord[] coords = new Coord[4];
    coords[0] = new Coord(-width/2,-height/2);
    coords[1] = new Coord(+width/2,-height/2);
    coords[2] = new Coord(+width/2,+height/2);
    coords[3] = new Coord(-width/2,+height/2);
    return coords;
  }

  public static BluePrint standardBluePrint(int width, int height){
    Coord[] coords = rectangle(width, height);
    return new BluePrint(coords, false);
  }

  public static PolygonStructure[] city(BluePrint[] prints, int houseCount, int x, int y, int citySize){
    PolygonStructure[] houses = new PolygonStructure[houseCount];
    for(int i = 0; i<houses.length; i++){
      houses[i] = randomHouse(prints, x, y, citySize);
    }
    return houses;
  }

  public static PolygonStructure randomHouse(BluePrint[] prints, int x, int y, int distance){
    double angle = generator.nextFloat()*2*Math.PI;
    int dist = generator.nextInt(distance);
    int structureX = x+(int)(dist*Math.cos(angle));
    int structureY = y+(int)(dist*Math.sin(angle));
    return new PolygonStructure(
      prints[generator.nextInt(prints.length)],
      generator.nextFloat()*2*Math.PI,
      structureX,
      structureY);
  }

  public static BluePrint[] randomBluePrints(int count, int minSize, int maxSize){
    BluePrint[] prints = new BluePrint[count];
    for(int i = 0; i < prints.length; i++){
      prints[i] = randomBluePrint(minSize, maxSize);
    }
    return prints;
  }

  public static BluePrint randomBluePrint(int minSize, int maxSize){
    int spread = maxSize - minSize;
    return standardBluePrint(generator.nextInt(spread)+minSize, generator.nextInt(spread)+minSize);
  }

}
