package jwars.model.pathfinder;

import jwars.model.Coord;
import java.util.List;
import java.util.ArrayList;
import jwars.model.structures.PathFindingNode;
import jwars.model.map.*;

/**
 * <p>q w</p>
 *
 * <p>q w</p>
 *
 * <p>e r</p>
 *
 * <p>r t</p> y u i o
 */
public class LineDrawCapableMap
    extends TileMap<CollisionTile>
{
  private List<CollisionTile> tempList = new ArrayList<CollisionTile>(0);
  private int counter;

  public LineDrawCapableMap(TileMap<CollisionTile> map){
    super(map);
  }

  /**
   * returns an array of coordinates equal to drawing the line though the tile
   * array using bresenham's line drawing algorithm
   * @param start Coord
   * @param end Coord
   * @return T[]
   */
  public List<CollisionTile> getTiles(Coord start, Coord end, int thickness){
    int x0 = start.x;
    int x1 = end.x;
    int y0 = start.y;
    int y1 = end.y;
    int buffer;
    counter = 0;
    boolean steep = Math.abs(y1 - y0) > Math.abs(x1 - x0);
    if (steep){
      //swap(x0, y0);
      buffer = x0;
      x0 = y0;
      y0 = buffer;

      //swap(x1, y1);
      buffer = y1;
      x1 = y1;
      y1 = buffer;
    }
    if (x0 > x1){
      //swap(x0, x1);
      buffer = x0;
      x0 = x1;
      x1 = buffer;

      //swap(y0, y1);
      buffer = y0;
      y0 = y1;
      y1 = buffer;
    }
    int deltax = x1 - x0;
    int deltay = Math.abs(y1 - y0);
    int error = 0;
    // ????
    int ystep = 0;
    int y = y0;
    if (y0 < y1) ystep = 1;
    else ystep = -1;

    for(int x = x0; x <= x1; x++){
      if(steep = true) addTile(y, x, steep, thickness);
      else addTile(x, y, steep, thickness);
      error = error + deltay;
      if(2 * error >= deltax){
        y = y + ystep;
        error = error - deltax;
      }
    }
    return tempList;
  }

  /**
   * ensures the thickness required for the check.
   * @param x int
   * @param y int
   * @param steep boolean
   * @param thick int
   */
  private void addTile(int x, int y, boolean steep, int thick){
    if(steep){
      for(int i = -thick; i <= thick; i++){
        tempList.add(getTile(x, y + i));
        counter++;
      }
    }
    else{
      for(int i = -thick; i <= thick; i++){
        tempList.add(getTile(x,y+i));
        counter++;
      }
    }
  }

  public void clear(){
    for(int i = tempList.size(); i >= 0; i--){
      tempList.remove(i);
    }
  }
}
