package jwars.model.pathfinder;

import jwars.model.structures.TerrainObject;
import java.util.Arrays;
import jwars.model.map.*;
import jwars.model.Coord;
import java.util.*;
import jwars.model.Collider;
import jwars.model.structures.*;
import jwars.model.pathfinder.LineDrawCapableMap;
import jwars.model.Unit;

public class Pathfinder{

  private BluePrint bluep;
  private PathFindingNode start;
  private TerrainObject end;
  private TerrainObject col;

  private TileMap colMap;
  private int listSize = 100;
  private boolean weighted;
  private int steps;
  private int thickness;
  private int unitRadius;
  private boolean succes;

  private LineDrawCapableMap ldcMap;
  private List<CollisionTile> tilesPassed;

  private NodeList openList;
  private NodeList closedList;

  public Pathfinder(TileMap<CollisionTile> colMap, boolean weighted){
    this.weighted = weighted;

    bluep = new BluePrint(true, 1);
    end = new TerrainObject(bluep, 0, 0, 0);

    ldcMap = new LineDrawCapableMap(colMap);
    openList = new NodeList(listSize);
    closedList = new NodeList(listSize);

  }

  public void findPath(Coord endc, Unit unit){
    PathFindingNode current;
    unitRadius = unit.getRadius();
    thickness = unitRadius; //udregn thickness fra maincord/ldcMapsize
    //(den strste)+1 og del med 2;

    ldcMap.getTileAt(end.getX(), end.getY()).removeT(end);
    this.end.location.set(unit.getX(), unit.getY());
    ldcMap.getTileAt(unit.getX(), unit.getY()).registerT(end);

    start.set(unit.getX(), unit.getY());
    openList.addNode(start);
    closedList.addNode(start);
    current = openList.popHead();

    do{
      expand(current, end.getStructPointZero(), unit);
      openList.sort();
      current = openList.popHead();
    } while(openList.isEmpty() != true);

    succes = false;
    System.out.println("FEJL FEJL FEJL");
  }

  private void expand(PathFindingNode from, PathFindingNode to, Unit unit){
    PathFindingNode expand1 = null;// = from.calculateDestinationFromCorner(unit.getRadius());
    PathFindingNode expand2 = null;// = to.calculateDestinationFromCorner(unit.getRadius());
    Coord startCoord = from.calculateDestinationFromCorner(unit.getRadius());
    Coord endCoord = to.calculateDestinationFromCorner(unit.getRadius());
    List<CollisionTile> tiles = ldcMap.getTiles(startCoord, endCoord, 0);

    double max = 0;
    double min = 0;
    for(int i = 0; i < tiles.size(); i++){
      // Walking through from behind to get the latest addition
      for(int j = tiles.get(i).colliders.size()-1; j >= 0; j--){
          // find min + max PFN's
        if(tiles.get(i).colliders.get(i) == end){
          backtrack(unit);
          clean();
        }


        if(expand1 == to){
          // skal have indfrt terrainobjects istedet for en collider list
          expand1.update(from);
          openList.addNode(expand1);
          closedList.addNode(expand1);
        }
        else{
          expand(from, expand1, unit);
        }
        if(expand2 == to){
          expand2.update(from);
          openList.addNode(expand2);
          closedList.addNode(expand2);
        }
        else{
          expand(from, expand2, unit);
          expand1.calculateDestinationFromCorner(unit.getRadius());
        }
      }
    }
  }

  /**
   * Used for first backtracking through the pathfindingnodes setting the baby
   * at each pass and then tracking through appending the moves on the
   * selected unit
   * @param unit Unit
   */
  private void backtrack(Unit unit){
    PathFindingNode current = end.getStructPointZero();
    int counter = 0;
    do{
      current.ancestor.baby = current;
      current = current.ancestor;
      counter++;
    } while(current != start);
    do{
      unit.appendMove(current.x, current.y);
      current = current.baby;
    } while(current != end.getStructPointZero());
    // skal displaces
    unit.appendMove(end.location.x, end.location.y);
  }



  private void clean(){
    closedList.reset();
    openList.clear();
    closedList.clear();
  }

  /**
   * The internal class NodeList is needed for containing PathFdindingNodes
   * for the pathfinder. In the algorithm used we need both the closedList
   * and the openList.
   */
  private static class NodeList{
    final PathFindingNode[] node;
    int count;
    boolean f_sorted;

    public NodeList(int size){
      node = new PathFindingNode[size];
    }

    // Add a node. Returns false if there's no room.
    boolean addNode(PathFindingNode newNode){
      if(count == node.length)
        return false;
      f_sorted = false;
      node[count++] = newNode;

      return true;
    }

    // Remove a node from the list
    void removeNode(Coord co){
      int index = findNode(co);
      assert index != -1;
      f_sorted = false;
      node[index] = node[--count];
      node[count] = null;
    }

    void sort(){
      Arrays.sort(node);
      f_sorted = true;
    }

    // Find a node's position by its coordinates. Returns -1 if node is not found.
    int findNode(Coord newcoord){

      for(int i = 0; i < count; i++)
        if(node[i].equals(newcoord))
          return i;

      return -1;
    }

    // Return a node by its coordinates (or null if not present)
    PathFindingNode getNode(Coord coord){
      int index = findNode(coord);
      if(index == -1)
        return null;
      else
        return node[index];
    }

    // clear the list - used for resetting the pathfinder
    void clear(){
      for(int i = 0; i < count; i++)
        node[i] = null;
      count = 0;
    }

    int size(){
      return count;
    }

    void reset(){
      for(int i = 0; i <= node.length; i++){
        node[i].reset();
      }
    }

    boolean isEmpty(){
      return count == 0;
    }

    // Pop the head of the list
    PathFindingNode popHead(){
      if(count > 0 && !f_sorted){
        Arrays.sort(node, 0, count);
        f_sorted = true;
      }
      PathFindingNode ret = node[0];
      node[0] = node[--count];
      node[count] = null;
      return ret;
    }

  }

  /**
   *
                                  2                         2
   qx r1x - qx r0x - r0x r1x + r0x  - qy r0y - r0y r1y + r0y  + qy r1y
   -------------------------------------------------------------------
             2                  2      2                  2
          r1x  - 2 r0x r1x + r0x  + r1y  - 2 r0y r1y + r0y
   */
  public static double pointLineDist(Coord src, Coord dst, Coord point){
    double
        qx = point.x, qy = point.y,
        r0x = src.x, r0y = src.y,
        r1x = dst.x, r1y = dst.y;
    double enumerator = qx*(r1x-r0x) - r0x*r1x + r0x*r0x - qy*(r0y-r1y) - r0y*r1y + r0y*r0y;
    double denominator = (r1x-r0x)*(r1x-r0x) + (r1y-r0y)*(r1y-r0y);
    double t = enumerator / denominator;
    double x = r0x + t*(r1x-r0x);
    double y = r0y + t*(r1y-r1x);
    double distSqr = (x-qx)*(x-qx) + (y-qy)*(y-qy);
    return Math.sqrt(distSqr);
  }

}
