package jwars.model.structures;

import jwars.gui.units.Sprite;
import jwars.model.Collider;
import jwars.model.CollisionProperties;
import jwars.model.Coord;
import java.awt.Graphics2D;
import jwars.model.WarModel;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import jwars.model.structures.PathFindingNode;

public class TerrainObject
    implements Collider{

  private Image image;

  public PathFindingNode[] structPoints;
  public Coord location = new Coord();
  private TerrainSprite sprite;
  private CollisionProperties collisionProps;
  private int radius;
  private double angle;
  public boolean small;

  public TerrainObject(Image image, BluePrint bluePrint, double angle, int x,
                       int y){

    this.angle = angle;
    this.image = image;
    this.radius = bluePrint.getRadius();
    if(radius < 500){
      small = true;
    } else small = false;

    createPathFindingNodes(bluePrint.convexHull);

    this.location.set(x, y);
   // radius = 0;
    collisionProps = new CollisionProperties(bluePrint.getMassive(),
        bluePrint.getMoveable());

    for(PathFindingNode c : structPoints){
      c.rotate(angle).plus(location);
    }

    sprite = new TerrainSprite(getRadius());
  }
  // used for end location
  public TerrainObject(BluePrint bluePrint, double angle, int x,
                       int y){
    this.angle = angle;
    this.radius = bluePrint.getRadius();
    if(radius < 500){
      small = true;
    } else small = false;
    this.location.set(x, y);
  }

  private void createPathFindingNodes(Coord[] points){
    System.out.println("points length = "+points.length+"");
    Coord lastSegment=new Coord(), nextSegment = new Coord(),
        indentDirection = new Coord();
    for(int i = 0; i < points.length-1; i++){
      System.out.println("trying to make one");
      structPoints[i] = new PathFindingNode(points[i]);
      lastSegment.set(points[i]);
      if(i>0){
        lastSegment.minus(points[i - 1]).rotate(Math.PI / 2); //let's hope the points
        //are traversed the right way, otherwise there's gonna be an error
      } else {
        lastSegment.minus(points[points.length - 1]).rotate(-Math.PI/2);
      }
      nextSegment.set(points[(i+1) % points.length]);
      nextSegment.minus(points[i]).rotate(-Math.PI/2);
      indentDirection.set(0,0);
      indentDirection.plus(lastSegment).plus(nextSegment);
      double unNormalizedIndentDirectionLength = indentDirection.length();
      double indentX = indentDirection.x/unNormalizedIndentDirectionLength;
      double indentY = indentDirection.y/unNormalizedIndentDirectionLength;
      structPoints[i].setIndentDirection(indentX,indentY);
      //we still haven't extended it to allow objects to pass sharp corners
      System.out.println("made one");
    }
  }

  public PathFindingNode getStructPointZero(){
    return structPoints[0];
  }

  public CollisionProperties getCollisionProps(){
    return collisionProps;
  }

  public int getX(){
    return location.x;
  }

  public int getY(){
    return location.y;
  }

  public int getRadius(){
    return radius;
  }

  public Sprite getSprite(){
    return sprite;
  }

  public void moveObject(Coord newC){
    location.set(newC);
  }


  public void collisionDetected(Collider other){
  }

  private class TerrainSprite
      extends Sprite{

    public TerrainSprite(int radius){
      super(radius);

      /*   xPoints = new int[structPoints.length];
         yPoints = new int[structPoints.length];
         for(int i = 0; i < structPoints.length; i++){
           xPoints[i] = structPoints[i].x - location.x;
           yPoints[i] = structPoints[i].y - location.y;
         }
         WarModel.WORLD.MAIN_COORDS.transformToCoords(WarModel.PIXEL_COORDS,
             xPoints);
         WarModel.WORLD.MAIN_COORDS.transformToCoords(WarModel.PIXEL_COORDS,
             yPoints);*/
    }

    public void draw(Graphics2D g2d, int x, int y){
      g2d.rotate(angle, x, y);
      g2d.drawImage(image, x, y, null);
    }

  }
}
