package jwars.model.units;

import java.awt.Color;
import java.awt.Graphics2D;

import jwars.data.InfantryCategory;
import jwars.gui.units.Sprite;
import jwars.model.Moveable;
import jwars.model.WarModel;
import jwars.model.World;

public class InfantrySquad extends Moveable{

  //private static int radiusInPixels = 18;
  //private static int radiusInUnits = WarModel.PIXEL_COORDS.transformToCoords(
  //    WarModel.WORLD.MAIN_COORDS, radiusInPixels);

  private final static Color camo = Color.GREEN.darker().darker().darker();
  private final static Color feldGrau = Color.darkGray;

  private final static int[] xLocations = {0, -4, 5, 1, -8,  6,  -7,  8,   0, 7, 10, -4};
  private final static int[] yLocations = {0, -6, 3, 7, -2, -6,  7,  -2, -10, 8,  3,  9};

  //private final static String infantryName = "Rifle sqd";
  private int strength;
  protected final InfantryCategory CAT;

  private Color uniformColor;

  public InfantrySquad(InfantryCategory cat){
    super(cat, false);
    this.CAT = cat;
    if(cat.squadSize > xLocations.length){
      throw new IllegalArgumentException("Illegal squad size: "+cat.squadSize+". Maximum allowed: "+xLocations.length);
    }
    this.strength = cat.squadSize;
    //super(infantryName, radiusInUnits, 12*kilometer/hour, false);
    setSprite(new DefaultSprite());
  }

  public void addToWorld(World world, int x, int y){
    super.addToWorld(world, x, y);
    if(this.subordinateOf(Teams.SOVIET_UNION)){
      uniformColor = camo;
    } else if(this.subordinateOf(Teams.GERMANY)){
      uniformColor = feldGrau;
    } else {
      uniformColor = Color.cyan;
    }
  }

  public int getStrength(){
    return strength;
  }

  //private static int actualRadiusInPixels = radiusInPixels;// - 4;

  private class DefaultSprite extends Sprite{

    public DefaultSprite(){
      super(WarModel.WORLD.MAIN_COORDS.transformToCoords(WarModel.PIXEL_COORDS, CAT.radius));
    }

    public void draw(Graphics2D g, int x, int y){
      for(int i = 0; i < strength; i++){
        drawSoldier(g, x+xLocations[i], y+yLocations[i]);
      }
    }

    private void drawSoldier(Graphics2D g, int x, int y){
      g.setColor(uniformColor);
      g.fillRect(x-1, y, 2, 4); //Body
      g.setColor(Color.yellow);
      g.drawLine(x, y-1, x, y-1); //Head
      g.setColor(Color.black);
      g.drawLine(x-3, y, x+1, y+2); //Gun
      g.drawLine(x, y-2, x, y-2); //Helmet
    }

  }

}
