package jwars.model.vision;

import jwars.event.Event;
import jwars.test.Link;

public class ObserverModel
    extends Link<ObserverModel>{

  final ObservationEnvironment env;
  final Event event;
  final Observer subject;
  ObserverTile generalLocation;

  public ObserverModel(ObservationEnvironment env, Observer subject){
    super.setValue(this);
    this.env = env;
    this.event = new ObservationEvent();
    this.subject = subject;
  }

  protected void performCompleteCheck(int dt){
    //The environment knows how the tile layout looks and will kindly
    //invoke the check methods of this object appropriately
    env.traverseSurroundingTiles(this, dt);
  }

  protected void performTileCheck(ObserverTile tile, int dt){
    //Traverse tile, perform check on each element
    for(Link<ObserverModel> link : tile){
      ObserverModel obs = link.value();
      if(!subject.sharesVision(obs.subject)){
        performCheck(obs, dt);
      }
    }
  }

  /*public boolean sharesVision(ObserverModel other){
    if(other == this){
      return true;
    }
    return true; //implement later
  }*/

  protected boolean performCheck(ObserverModel other, int dt){
    float dx = (float)other.subject.getX()-this.subject.getX();
    float dy = (float)other.subject.getY()-this.subject.getY();
    float distSqr = dx*dx + dy*dy;
    float visibilitySqr = (float)env.maxVisibility * env.maxVisibility;
    return (distSqr < visibilitySqr) && subject.visionCheck(other.subject, (int)Math.sqrt(distSqr), dt);
  }

  private class ObservationEvent
      extends Event{

    private int lastCheckTime = 0;

    public ObservationEvent(){
      super(env.getObservationQueue());
    }

    protected void execute(){
      int currentTime = env.timer.getTime();
      int timeChange = currentTime - lastCheckTime;
      lastCheckTime = env.timer.getTime();
      performCompleteCheck(timeChange);
      enqueue();
    }

  }

}
