package jwars.model.weapons;

import java.util.ArrayList;
import java.util.List;

import jwars.data.WeaponCategory;

public class WeaponModel implements ReloadListener {

  private final List<Weapon> weapons = new ArrayList<Weapon>(0);
  private boolean canFire;
  private ReloadListener listener;

  public WeaponModel(ReloadListener listener, List<WeaponCategory> weaponry){
    for(WeaponCategory cat : weaponry){
      Weapon w = cat.create();
      this.listener = listener;
      w.setReloadListener(this);
      weapons.add(w);
    }
    canFire = isAtLeastOneWeaponReady();
  }

  private boolean isAtLeastOneWeaponReady(){
    for(Weapon w : weapons){
      if(w.isLoaded())
        return true;
    }
    return false;
  }

  public void fire(int srcX, int srcY, int dstX, int dstY){
    for(Weapon w : weapons){
      if(w.isLoaded()){
        w.fire(srcX, srcY, dstX, dstY);
      }
    }
    //canFire = false;
  }

  public boolean reportReloaded(Weapon weapon){
    canFire = true;
    return true;//listener.reportReloaded(weapon);
  }

  public boolean canFire(){
    return canFire;
  }

}
