package jwars.net;

import java.io.DataInput;
import java.io.DataInputStream;
import java.io.DataOutput;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Collection;
import java.util.Hashtable;

import jwars.model.units.Teams;
import jwars.test.Player;
import jwars.Util;

public class DefaultServerEventHandler
    implements ServerEventHandler{

  private Hashtable<Integer, Player> players = new Hashtable<Integer, Player>();

  public void clientRemoved(int clientNumber, DataInputStream clientInput,
                            DataOutputStream clientOutput,
                            DataOutputStream commonOutput) throws IOException{
    //Deregister client
    Player removed = players.remove(clientNumber);
    if(removed == null){
      System.out.println("DefaultServerEventHandler: Nonexistent client "+removed+" specified for removal");
    } else {
      System.out.println("DefaultServerEventHandler: Client "+removed+" [id="+clientNumber+"] has been removed");
    }
    //Inform others that this client is to be removed
    //commonOutput.writeInt(clientNumber);
  }

  public boolean handShake(int clientNumber, DataInputStream clientInput,
                        DataOutputStream clientOutput) throws IOException{
    //Read info from client
    String version = clientInput.readUTF();
    if(!version.equals(Util.VERSION)){
      System.out.println("Joining client has conflicting version, disconnecting");
      System.out.println("Server version: "+Util.VERSION);
      System.out.println("Client version: "+version);
      System.out.println("(Note: versions earlier than alpha 2 build 6 will send meaningless version data due to protocol modifications)");
      return false;
    }
    String name = clientInput.readUTF();
    String team = clientInput.readUTF();
    //Send existing info to client
    Collection<Integer> playerIndices = players.keySet();
    clientOutput.writeInt(playerIndices.size());
    for(Integer i : playerIndices){
      clientOutput.writeInt(i);
      Player p = players.get(i);
      clientOutput.writeUTF(p.getName());
      clientOutput.writeUTF(p.getTeam().NAME_SHORT);
    }
    clientOutput.flush();
    players.put(clientNumber, new Player(name, Teams.getTeamByShortName(team)));
    return true;
  }

  public void clientJoined(int clientNumber, DataOutputStream commonOutput) throws IOException{
    //Register client and write to all others that this client joined
    Player p = players.get(clientNumber);
    commonOutput.writeUTF(p.getName());
    commonOutput.writeUTF(p.getTeam().NAME_SHORT);
    commonOutput.flush();
  }

  public void clientPropertyChanged(int clientNumber,
      DataInput clientInput,
      DataOutput commonOutput) throws IOException{
    Player p = players.get(clientNumber);
    String name = clientInput.readUTF();
    String team = clientInput.readUTF();
    p.setName(name);
    p.setTeam(Teams.getTeamByShortName(team));
    commonOutput.writeInt(clientNumber);
    commonOutput.writeUTF(name);
    commonOutput.writeUTF(team);
  }

}
