package jwars.net;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.ArrayList;

public class Protocol{

  private final static int max_id = Byte.MAX_VALUE;
  private int currentId = Byte.MIN_VALUE;
  private ArrayList<IOHandler> types = new ArrayList<IOHandler>();
  private IOHandler keepAliveInstruction;

  DataOutputStream output;
  Server server;
  Client client;

  public Protocol(IOHandler keepAliveInstruction){
    this.keepAliveInstruction = keepAliveInstruction;
    addType(this.keepAliveInstruction);
    addType(this.clientJoined);
    addType(this.clientLeft);
    addType(this.clientPropChng);
  }

  public void addType(IOHandler serializer){
    if(currentId == Byte.MAX_VALUE)
      throw new RuntimeException("Can register only " + max_id + " elements");
    byte uniqueId = (byte)currentId++;
    if(serializer.uniqueId != serializer.unAssigned)
      throw new IllegalArgumentException("Serializer already registered");
    serializer.uniqueId = uniqueId;
    serializer.output = output;
    types.add(serializer);
  }

  public void setOut(DataOutputStream output){
    this.output = output;
    for(IOHandler s : types)
      s.output = output;
  }

  public IOHandler getType(byte id){
    return types.get(id - Byte.MIN_VALUE);
  }

  /*public void setKeepAliveInstruction(Serializer serializer){
    serializer.output = output;
    serializer.uniqueId = Byte.MIN_VALUE;
    types.set(0, serializer);
     }*/

  public IOHandler getKeepAliveInstruction(){
    return this.keepAliveInstruction;
  }

  /*public IOHandler getClientJoinInstruction(){
    return this.clientJoinInstruction;
     }

     public IOHandler getClientLeaveInstruction(){
    return this.clientLeaveInstruction;
     }

     public IOHandler getHandShakeInstruction(){
    return this.handShakeInstruction;
     }*/

  private static class DefaultKeepAliveInstruction
      extends IOHandler{

    protected void read(DataInput input, int timeStamp) throws IOException{
    }

    protected void echo(int clientIdentifier, DataInput input,
                        DataOutput output) throws IOException{
    }

    protected void write(Object t, DataOutput output) throws IOException{
    }

  }

  IOHandler clientJoined = new ClientJoinInstruction();

  private class ClientJoinInstruction
      extends IOHandler{

    protected void read(DataInput input, int timeStamp) throws IOException{
      client.eventHandler.clientJoined(input.readInt(), input);
    }

    protected void echo(int clientIdentifier, DataInput input,
                        DataOutput output) throws IOException{
      output.writeInt(clientIdentifier);
    }

    protected void write(Object t, DataOutput output) throws IOException{
    }

  }

  IOHandler clientLeft = new ClientLeftInstruction();

  private class ClientLeftInstruction
      extends IOHandler{

    protected void read(DataInput input, int timeStamp) throws IOException{
      client.eventHandler.clientRemoved(input.readInt(), input);
    }

    protected void echo(int clientIdentifier, DataInput input,
                        DataOutput output) throws IOException{
      output.writeInt(clientIdentifier);
    }

    protected void write(Object t, DataOutput output) throws IOException{
    }

  }

  IOHandler clientPropChng =
      new ClientPropertyChangeInstruction();

  private class ClientPropertyChangeInstruction
      extends IOHandler{

    protected void read(DataInput input, int timeStamp) throws IOException{
      client.eventHandler.clientPropertyChanged(input.readInt(), input);
    }

    protected void echo(int clientIdentifier, DataInput input,
                        DataOutput output) throws IOException{
      server.eventHandler.clientPropertyChanged(clientIdentifier, input, output);
    }

    protected void write(Object t, DataOutput output) throws IOException{
      client.eventHandler.writeClientProperties(output);
    }

  }

}
