package jwars.test;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintStream;
import java.util.Properties;

public class JWProperties
    extends Properties{

  //private final static Properties defaults;// = new Properties();
  //private final Properties props;// = new Properties();
  private final File file;

  public final static String NAME = "name";
  public final static String RESOLUTION = "resolution";
  public final static String PORT = "port";
  public final static String LATENCY = "latency";

  public JWProperties(String fileName){
    super(defaultProperties());
    file = new File(fileName);
    InputStream in = null;
    if(file.exists()){
      try{
        in = new FileInputStream(file);
        load(in);
        in.close();
      } catch(FileNotFoundException ex){
        System.err.println("File " + fileName +
                           " not found, using default settings");
        ex.printStackTrace();
      } catch(IOException ex){
        System.err.println("Error loading settings from " + fileName +
                           ", will use default settings");
        ex.printStackTrace();
      } finally{
        if(in != null){
          try{
            in.close();
          } catch(IOException ex){
            System.err.println("Could not close input stream from file " +
                fileName);
            ex.printStackTrace();
          }
        }
      }
    }
  }

  public Object setProperty(String key, String value){
    System.out.println("Property \""+key+"\" set to \""+value+"\"");
    return super.setProperty(key, value);
  }

  public boolean store(){
    System.out.println("Saving configuration in "+file);
    file.delete();
    try{
      file.createNewFile();
    } catch(IOException ex){
      System.err.println("Could not create file "+file);
      ex.printStackTrace();
      return false;
    }
    OutputStream out = null;
    try{
      out = new FileOutputStream(file);
    } catch(FileNotFoundException ex){
      System.err.println("Could not create output stream for "+file);
      ex.printStackTrace();
      return false;
    }
    try{
      super.store(out, "JWars property file");
    } catch(IOException ex){
      System.err.println("Could not store property list in "+file);
      ex.printStackTrace();
      return false;
    }
    try{
      out.close();
    } catch(IOException ex){
      System.err.println("Could not close file output stream for "+file);
      ex.printStackTrace();
    }
    return true;
  }

  public void printDefaults(PrintStream out){
    java.util.Enumeration names = defaults.propertyNames();
    while(names.hasMoreElements()){
      Object elem = names.nextElement();
      System.out.println(elem + "="+defaults.get(elem));
    }
    /*try{
      Enumeration x = defaults.propertyNames();
      for(Object o : defaults.propertyNames()){

      }
      //defaults.store(out, null);
    } catch(IOException ex){
      ex.printStackTrace();
    }*/
  }

  private static Properties defaultProperties(){
    Properties p = new Properties();
    p.setProperty(NAME, "Manstein");
    p.setProperty(RESOLUTION, "800x600");
    p.setProperty(PORT, "7777");
    p.setProperty(LATENCY, "0.2");
    return p;
  }

}
