package jwars.test;

import java.util.Iterator;

public class LinkList<T>
    implements Iterable<Link<T>>{

  private final Link<T>
      start = new FixedLink(),
      end = new FixedLink();

  public LinkList(){
    start.next = end;
    start.previous = null;
    end.next = null;
    end.previous = start;
  }

  public void enqueue(Link<T> link){
    link.insertAfter(end.previous);
  }

  public Link dequeue(){
    Link returnValue = start.next;
    if(returnValue == end){
      return null;
    }
    assert(returnValue.previous == start && returnValue.next != null);
    returnValue.remove();
    return returnValue;
  }

  public Link pop(){
    Link returnValue = end.previous;
    if(returnValue == start){
      return null;
    }
    assert(returnValue.next == end && returnValue.previous != null);
    returnValue.remove();
    return returnValue;
  }

  public Iterator<Link<T>> iterator(){
    return new LinkIterator();
  }

  private class LinkIterator
      implements Iterator<Link<T>>{

    public Link<T> current = start;

    public boolean hasNext(){
      return current.next != end;
    }

    public Link<T> next(){
      if(current.next == end){
        throw new java.util.NoSuchElementException(
            "Iterator has no more elements!");
      }
      return current = current.next;
    }

    public void remove(){
      Link<T> previous = current.previous;
      current.remove();
      current = previous;
    }
  }

  public static void main(String[] args){
    LinkList<Object> chain = new LinkList<Object>();
    chain.enqueue(new Link<Object>());
    chain.enqueue(new Link<Object>());
    chain.pop();
    chain.enqueue(new Link<Object>());
    chain.dequeue();
    int i = 0;
    for(Link l : chain){
      i++;
    }
    System.out.println("" + i);
    chain.dequeue();
  }

  private class FixedLink
      extends Link<T>{

    protected void remove(){
      throw new UnsupportedOperationException("Error in LinkList, fixed links manipulated");
    }

    protected void insertAfter(Link<T> other){
      throw new UnsupportedOperationException("Error in LinkList, fixed links manipulated");
    }

    protected void insertBefore(Link<T> other){
      throw new UnsupportedOperationException("Error in LinkList, fixed links manipulated");
    }

  }

}
